local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables

-- SpellFlash API:
-- http://wow.curseforge.com/addons/spellflash/pages/api/

-- Updated variable references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-local-x-spell-flash-addon-updated-variables
-- x.Enemy, x.ActiveEnemy, x.NoCC, x.InInstance, x.InstanceType, x.PetAlive, x.PetActiveEnemy, x.PetNoCC, x.Lag, x.DoubleLag, x.ThreatPercent, x.EnemyDetected, x.ShouldPermanentBuff, x.ShouldTemporaryBuff

-- Spell table references:
-- http://wow.curseforge.com/addons/spellflash/pages/api/#w-s-castable

local SpellPowerMultiplier = {
	1459--[[Arcane Brilliance]],
	77747--[[Burning Wrath]],
	109773--[[Dark Intent]],
}

local MagicVulnerability = {
	1490--[[Curse of the Elements]],
	58410--[[Master Poisoner]],
	116202--[[Aura of the Elements]],
}

local function time_to_die()
	if s.MaxHealth("target") > 0 then
	-- Some very basic assumptions
		if s.InRaid() then
			if s.Boss("target") then
			-- Assume raid bosses to live a bout 5 minutes
				return s.HealthPercent("target") * 3
			else
				return s.HealthPercent("target") * 0.5
			end
		else
			if s.Boss("target") then
				return s.HealthPercent("target") * 1.2
			else
				return s.HealthPercent("target") * 0.1
			end
		end
	else
		return 31
	end
end

a.spells["Auto Attack"] = {
	ID = 6603--[[Auto Attack]],
	EnemyTargetNeeded = 1,
	Melee = 1,
	NotIfActive = 1,
	CheckFirst = function(z)
		z.FlashColor = s.If(x.ActiveEnemy, "White", "Pink")
		return x.NoCC
	end,
}

a.spells["Shoot"] = {
	ID = 3018--[[Shoot]],
	EnemyTargetNeeded = 1,
	NotIfActive = 1,
	CheckFirst = function(z)
		z.FlashColor = s.If(x.ActiveEnemy, "White", "Pink")
		return x.NoCC and not s.Shooting()
	end,
}

a.spells["Dark Intent"] = {
	ID = 109773--[[Dark Intent]],
	Buff = SpellPowerMultiplier,
	EarlyRefresh = 10,
	NoRangeCheck = 1,
	BuffUnit = "raid",
	FlashColor = "Green",
}

a.spells["Curse of the Elements"] = {
	ID = 1490--[[Curse of the Elements]],
	Debuff = MagicVulnerability,
	EarlyRefresh = 5,
	EnemyTargetNeeded = 1,
	CheckFirst = function(z)
		return x.NoCC
	end,
}

a.spells["Summon Felhunter ooc"] = {
	ID = 691--[[Summon Felhunter]],
	CheckFirst = function(z)
		return not s.InCombat() and not x.PetAlive
	end,
	FlashID = {691--[[Summon Felhunter]], 112869--[[Summon Observer]]}
}

a.spells["Summon Felguard ooc"] = {
	ID = 30146--[[Summon Felguard]],
	CheckFirst = function(z)
		return not s.InCombat() and not x.PetAlive
	end,
	FlashID = {30146--[[Summon Felguard]], 112870--[[Summon Wrathguard]]}
}

a.spells["Dark Soul: Misery"] = {
	ID = 113860--[[Dark Soul: Misery]],
	CheckFirst = function(z)
		return s.InCombat()
	end,
	FlashColor = "Yellow",
	NoGCD = 1,
	Continue = 1,
}

a.spells["Dark Soul: Knowledge"] = {
	ID = 113861--[[Dark Soul: Knowledge]],
	CheckFirst = function(z)
		return s.InCombat() and s.Power("player", SPELL_POWER_DEMONIC_FURY) >= 350
	end,
	FlashColor = "Yellow",
	NoGCD = 1,
	Continue = 1,
}

a.spells["Grimoire: Felhunter"] = {
	ID = 108501--[[Grimoire of Service]],
	CheckFirst = function(z)
		return s.InCombat() and IsPlayerSpell(108501--[[Grimoire of Service]])
	end,
	FlashColor = "Yellow",
	FlashID = {111897--[[Grimoire: Felhunter]], 108501--[[Grimoire of Service]]},
	NoGCD = 1,
	Continue = 1,
}

a.spells["Grimoire: Felguard"] = {
	ID = 108501--[[Grimoire of Service]],
	CheckFirst = function(z)
		return s.InCombat() and IsPlayerSpell(108501--[[Grimoire of Service]])
	end,
	FlashColor = "Yellow",
	FlashID = {111898--[[Grimoire: Felguard]], 108501--[[Grimoire of Service]]},
	NoGCD = 1,
	Continue = 1,
}

a.spells["Grimoire of Sacrifice"] = {
	ID = 108503--[[Grimoire of Sacrifice]],
	CheckFirst = function(z)
		return s.InCombat() and IsPlayerSpell(108503--[[Grimoire of Sacrifice]])
	end,
	FlashColor = "Yellow",
	NoGCD = 1,
	Continue = 1,
}

a.spells["Summon Felhunter (Sacrifice down)"] = {
	ID = 691--[[Summon Felhunter]],
	CheckFirst = function(z)
		return s.InCombat() and IsPlayerSpell(108503--[[Grimoire of Sacrifice]])  and s.MyBuffDuration(108503--[[Grimoire of Sacrifice]], "player") == 0 and not x.PetAlive
	end,
}

a.spells["Summon Doomguard"] = {
	ID = 18540--[[Summon Doomguard]],
	CheckFirst = function(z)
		return s.InCombat()
	end,
}

a.spells["Soulburn w/ Dark Soul"] = {
	ID = 74434--[[Soulburn]],
	Buff = 74434--[[Soulburn]],
	BuffUnit = "player",
	RequireMyBuff = 113860--[[Dark Soul: Misery]],
	RequireBuffUnit = "player",
	CheckFirst = function(z)
		duration = s.MyBuffDuration(113860--[[Dark Soul: Misery]], "player")
		return s.InCombat() and duration > 0 and (duration >= 18.5 or duration <= 1.5)
	end,
	EnemyTargetNeeded = 1,
	FlashColor = "Yellow",
	NoGCD = 1,
}

a.spells["Soul Swap w/ Soulburn"] = {
	ID = 86121--[[Soul Swap]],
	RequireMyBuff = 74434--[[Soulburn]],
	RequireBuffUnit = "player",
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Haunt"] = {
	ID = 48181--[[Haunt]],
	Debuff = 48181--[[Haunt]],
	EarlyRefresh = 1,
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC and s.Power("player", SPELL_POWER_SOUL_SHARDS ) > 1
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Agony"] = {
	ID = 980--[[Agony]],
	MyDebuff = 980--[[Agony]],
	EarlyRefresh = 2,
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC and time_to_die() >= 8
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Corruption"] = {
	ID = 172--[[Corruption]],
	MyDebuff = 172--[[Corruption]],
	EarlyRefresh = 3,
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC and time_to_die() >= 6
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Doom"] = {
	ID = 124913--[[Metamorphosis: Doom]],
	MyDebuff = 603--[[Doom]],
	EarlyRefresh = 30,
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Unstable Affliction"] = {
	ID = 30108--[[Unstable Affliction]],
	MyDebuff = 30108--[[Unstable Affliction]],
	EarlyRefresh = 3,
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC and time_to_die() >= 5
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Drain Soul as finisher"] = {
	ID = 1120--[[Drain Soul]],
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC and not s.NotDieing("target")
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Life Tap Affliction"] = {
	ID = 1454--[[Life Tap]],
	CheckFirst = function(z)
		return s.InCombat() and s.MaxPower("player", SPELL_POWER_MANA) > 0 and s.PowerPercent("player", SPELL_POWER_MANA) < 35
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Life Tap Demo"] = {
	ID = 1454--[[Life Tap]],
	CheckFirst = function(z)
		return s.InCombat() and not s.Buff(103958--[[Metamorphosis]], "player") and s.MaxPower("player", SPELL_POWER_MANA) > 0 and s.PowerPercent("player", SPELL_POWER_MANA) < 50
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Malefic Grasp"] = {
	ID = 103103--[[Malefic Grasp]],
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Fel Flame"] = {
	ID = 77799--[[Fel Flame]],
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Life Tap"] = {
	ID = 1454--[[Life Tap]],
	CheckFirst = function(z)
		return s.InCombat()
	end,
}

a.spells["Command Demon"] = {
	ID = 119915--[[Wrathstorm]],
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC and x.PetActiveEnemy and s.PetActionCooldown({115831--[[Wrathstorm]], 89751--[[Felstorm]]}) == 0
	end,
	FlashID = {119915--[[Wrathstorm]], 119914--[[Felstorm]]},
	Continue = 1,
}

a.spells["Felguard: Felstorm"] = {
	Type = "pet",
	ID = 115831--[[Wrathstorm]],
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC and x.PetActiveEnemy
	end,
	FlashID = {115831--[[Wrathstorm]], 89751--[[Felstorm]]},
}

a.spells["Metamorphosis"] = {
	Type = "form",
	ID = 103958--[[Metamorphosis]],
	NotIfActive = true,
	FlashBlink = true,
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC and (s.MyBuffDuration(113861--[[Dark Soul: Knowledge]], "player") > 0 or s.MyDebuffDuration(172--[[Corruption]], "target") < 5 or s.Power("player", SPELL_POWER_DEMONIC_FURY) >= 900 or s.Power("player", SPELL_POWER_DEMONIC_FURY) >= time_to_die()*30)
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Cancel Metamorphosis"] = {
	Type = "form",
	ID = 103958--[[Metamorphosis]],
	FlashBlink = true,
	Override = function(z)
		return s.InCombat() and s.Buff(103958--[[Metamorphosis]], "player") and ((s.MyDebuffDuration(172--[[Corruption]], "target") > 20) or (s.MyDebuffDuration(172--[[Corruption]], "target") == 0)) and s.MyBuffDuration(113861--[[Dark Soul: Knowledge]], "player") == 0 and s.Power("player", SPELL_POWER_DEMONIC_FURY) <= 750 and time_to_die() > 30
	end,
}

a.spells["Imp Swarm"] = {
	ID = 104316--[[Imp Swarm]],
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Hand of Gul'dan"] = {
	ID = 105174--[[Hand of Gul'dan]],
	MyDebuff = 47960--[[Shadowflame]],
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC and not s.SpellOrAuraDelay(105174--[[Hand of Gul'dan]], "target", any)
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Soul Fire"] = {
	ID = 6353--[[Soul Fire]],
	RequireMyBuff = 122355--[[Molten Core]],
	RequireBuffUnit = "player",
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC and (s.MyBuffStack(103958--[[Metamorphosis]], "player") == 0 or not s.NotDieing("target"))
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Doom"] = {
	ID = 124913--[[Metamorphosis: Doom]],
	MyDebuff = 603--[[Doom]],
	EarlyRefresh = 30,
	RequireBuff = 103958--[[Metamorphosis]],
	RequireBuffUnit = "player",
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC
	end,
	FlashID = {124913--[[Metamorphosis: Doom]], 603--[[Doom]], 172--[[Corruption]]},
	EnemyTargetNeeded = 1,
}

a.spells["Touch of Chaos"] = {
	ID = 112089--[[Metamorphosis: Touch of Chaos]],
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC
	end,
	EnemyTargetNeeded = 1,
}

a.spells["Shadow Bolt"] = {
	ID = 686--[[Shadow Bolt]],
	CheckFirst = function(z)
		return s.InCombat() and x.NoCC
	end,
	EnemyTargetNeeded = 1,
}